/*====================================================================

Voyage 200 Executables Patcher by Olivier Armand (ExtendeD) -
<ola.e-ml@wanadoo.fr>

current version : 0.12
Initial Date: 03/17/2002
Last Update : 02/27/2003

YOU ARE NOT ALLOWED TO USE OR MODIFY ANY PART OF THIS SOURCE CODE
WHITHOUT MY AUTHORIZATION.

VOUS N'ETES PAS AUTORISES A UTILISER OU MODIFIER CES SOURCES SANS
MON AUTORISATION.

Compile with the GCC Switch "-fno-function-cse"

=====================================================================*/

// ***************		Read the arguments, throw errors, create the memory
// * Entry point *    block, the sym entry, and decompress input file if
// ***************    necessary.

#define USE_V200
#define NO_EXIT_SUPPORT
#define NO_CALC_DETECT
#define MIN_AMS 208
#define USE_FLINE_ROM_CALLS

#include <tigcclib.h>
#include "ttunpack.h"

#if (defined(__TIGCC__VERSION) && (__TIGCC_VERSION__ < 94))
#error You need TIGCC v0.94 or higher to build this project
#endif
//(http://tigcc.ticalc.org)

#define fatal(s) do {ST_helpMsg(s); return;} while (0)
#define sfatal() fatal (buffer)

#define MAX_SYM_LEN (8*2+3)


unsigned short DoPatch (void* from, void* to)
									__attribute__ ((__regparm__(2)));

const char UsageStr[] = "usage: v200exep(\"infile\",\"outfile\")";
const char NoMemStr[] = "Not enough memory.";
const unsigned char PPG_SIGN[] = {0, 'p', 'p', 'g', 0, 0xF8};


void _main(void)
{
	ESI argptr = top_estack;
	unsigned char *in_name, *out_name;
	unsigned char *in, *out;
	HANDLE out_handle = H_NULL;
	unsigned char buffer[50];
	SYM_ENTRY *SymPtr;
	HSym out_sym;
	unsigned short patchs_num;
	unsigned long plen;


	if (ArgCount() != 2 || GetArgType(argptr) != STR_TAG)
		fatal (UsageStr);
		
	in_name = (unsigned char*)GetStrnArg (argptr);
	if (GetArgType(argptr) != STR_TAG)
		fatal (UsageStr);
	out_name = (unsigned char*)GetStrnArg (argptr);

	if ((TokenizeName(in_name, buffer)))
		fatal (UsageStr);
	if (!(SymPtr = SymFindPtr (buffer + MAX_SYM_LEN, 0)))
		{
			sprintf (buffer, "'%s' not found...", in_name);
			sfatal();
		}

	in = HeapDeref(SymPtr->handle);
	
	if ((TokenizeName (out_name, buffer)))
		fatal ("Invalid outfile name.");	
	if(SymFindPtr (buffer + MAX_SYM_LEN, 0))
		{
			sprintf (buffer, "'%s' already exists...", out_name);
			sfatal();
		}

	plen = (unsigned long)(*(unsigned short*)in + 2);
	if (*(in + plen - 1) != ASM_TAG)
		{
		  if (memcmp (in + plen - sizeof PPG_SIGN, PPG_SIGN, sizeof PPG_SIGN)
		      || !(ttunpack_valid (in + 2)))
		  	fatal ("Neither ASM nor PPG file...");
			plen = ttunpack_size(in + 2);
	  	if (!(out_handle = HeapAlloc (plen)))
	  		fatal (NoMemStr);
	  	out = HeapDeref (out_handle);
	  	if (ttunpack_decompress (in + 2, out))
				{
					HeapFree (out_handle);
					fatal ("Invalid PPG file.");
				}
		}
	
	else
		{
			if (!(out_handle = HeapAlloc (plen)))
				fatal (NoMemStr);
			out = HeapDeref (out_handle);
			memcpy (out, in, plen);
		}
		
	if (!(patchs_num = DoPatch (out + 2, out + plen + 1)))
		{
			sprintf (buffer, "'%s' doesn't need to be patched.", in_name);	
			HeapFree (out_handle);
			sfatal();
		}

  //'buffer' still contains the tokenized 'out_name'
	if (!((out_sym = SymAdd (buffer + MAX_SYM_LEN)).folder))
		{
			sprintf (buffer, "Error : can't create '%s'.", out_name);		
			sfatal();
		}
	
	(DerefSym (out_sym))->handle = out_handle;
	sprintf (buffer, "Patched program created - Patches applied: %i", patchs_num);
	sfatal();

}
